"""Tests for ClientCookie._HTTPDate."""

import sys, os, re, string, time
from unittest import TestCase


class DateTests(TestCase):
    def test_1(self):
        from ClientCookie._HTTPDate import parse_date, \
             str2time, time2str, time2iso, time2isoz

        from time import localtime, gmtime

        no = 1
        def ok(arg=0):
            global no
            if arg: print "not",
            print "ok %s" % (no,)
            no = no + 1


        # Tests added by JJL:

        base = 1019227000
        day = 24*3600
        assert time2str(base) == "Fri, 19 Apr 2002 14:36:40 GMT"
        assert time2str(base+day) == "Sat, 20 Apr 2002 14:36:40 GMT"
        assert time2str(base+2*day) == "Sun, 21 Apr 2002 14:36:40 GMT"
        assert time2str(base+3*day) == "Mon, 22 Apr 2002 14:36:40 GMT"


        # test str2time for supported dates.  Test cases with 2 digit year
        # will probably break in year 2044.
        tests = [
         'Thu Feb  3 00:00:00 GMT 1994',        # ctime format
         'Thu Feb  3 00:00:00 1994',            # same as ctime, except no TZ

         'Thu, 03 Feb 1994 00:00:00 GMT',       # proposed new HTTP format
         'Thursday, 03-Feb-94 00:00:00 GMT',    # old rfc850 HTTP format
         'Thursday, 03-Feb-1994 00:00:00 GMT',  # broken rfc850 HTTP format

         '03/Feb/1994:00:00:00 0000',   # common logfile format
         '03/Feb/1994:01:00:00 +0100',  # common logfile format
         '02/Feb/1994:23:00:00 -0100',  # common logfile format

         '03 Feb 1994 00:00:00 GMT',    # HTTP format (no weekday)
         '03-Feb-94 00:00:00 GMT',      # old rfc850 (no weekday)
         '03-Feb-1994 00:00:00 GMT',    # broken rfc850 (no weekday)
         '03-Feb-1994 00:00 GMT',       # broken rfc850 (no weekday, no seconds)
         '03-Feb-1994 00:00',           # VMS dir listing format

         '03-Feb-94',    # old rfc850 HTTP format    (no weekday, no time)
         '03-Feb-1994',  # broken rfc850 HTTP format (no weekday, no time)
         '03 Feb 1994',  # proposed new HTTP format  (no weekday, no time)
         '03/Feb/1994',  # common logfile format     (no time, no offset)

         #'Feb  3 00:00',     # Unix 'ls -l' format (can't really test it here)
         'Feb  3 1994',       # Unix 'ls -l' format

         "02-03-94  12:00AM", # Windows 'dir' format

         # ISO 8601 formats
         '1994-02-03 00:00:00 +0000',
         '1994-02-03',
         '19940203',
         '1994-02-03T00:00:00+0000',
         '1994-02-02T23:00:00-0100',
         '1994-02-02T23:00:00-01:00',
         '1994-02-03T00:00:00 Z',
         '19940203T000000Z',
         '199402030000',

         # A few tests with extra space at various places
         '  03/Feb/1994      ',
         '  03   Feb   1994  0:00  ',
        ]

        test_time = 760233600  # assume broken POSIX counting of seconds
        #print "time should be", test_time
        for test in tests:
            if string.find(string.upper(test), "GMT") != -1:
                t = str2time(test)
            else:
                t = str2time(test, "GMT")
            t2 = str2time(string.lower(test), "GMT")
            t3 = str2time(string.upper(test), "GMT")

            #print "'%s'  =>  %s" % (test, t)
            assert not (t is None or t != test_time or t2 != test_time or t3 != test_time)

        # test time2str
        assert time2str(test_time) == "Thu, 03 Feb 1994 00:00:00 GMT", "time2str failed"

        # test the 'ls -l' format with missing year$
        # round to nearest minute 3 days ago.
        test_time = int((time.time() - 3 * 24*60*60) /60)*60
        # XXX str2time timezone bug means we have to use gmtime, not localtime
        #mon, mday, hr, min = localtime(test_time)[1:5]
        mon, mday, hr, min = gmtime(test_time)[1:5]
        mon = ["Jan", "Feb", "Mar", "Apr", "May", "Jun",
               "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"][mon-1]
        text = "%s %02d %02d:%02d" % (mon, mday, hr, min)
        t = str2time(text)
        if t is None: t = "UNDEF"
        #print "'%s'  =>  %s (%s)" % (text, t, test_time)
        assert t == test_time

        # try some garbage.
        for test in (
            '', 'Garbage',
            'Mandag 16. September 1996',
        #     'Thu Feb  3 00:00:00 CET 1994',
        #     'Thu, 03 Feb 1994 00:00:00 CET',
        #     'Wednesday, 31-Dec-69 23:59:59 GMT',

            '1980-00-01',
            '1980-13-01',
            '1980-01-00',
            '1980-01-32',
            '1980-01-01 25:00:00',
            '1980-01-01 00:61:00',
            '1980-01-01 00:00:62'):
            bad = 0
            if str2time(test) is not None:
                print "str2time(%s) is not None" % (test,)
                print "str2time(test)", str2time(test)
                bad = bad + 1
            if test is None:
                pass
                #print "None"
            else:
                pass
                #print "'%s'" % (test)
            #print $@ if $@;
            assert not bad

        #print "Testing AM/PM gruff..."

        # Test the str2iso routines

        #print "Testing time2iso functions"

        t = time2iso(str2time("11-12-96  0:00AM"))#; print t
        assert t == "1996-11-12 00:00:00"

        t = time2iso(str2time("11-12-96 12:00AM"))#; print t
        assert t == "1996-11-12 00:00:00"

        t = time2iso(str2time("11-12-96  0:00PM"))#; print t
        assert t == "1996-11-12 12:00:00"

        t = time2iso(str2time("11-12-96 12:00PM"))#; print t
        assert t == "1996-11-12 12:00:00"


        t = time2iso(str2time("11-12-96  1:05AM"))#; print t
        assert t == "1996-11-12 01:05:00"

        t = time2iso(str2time("11-12-96 12:05AM"))#; print t
        assert t == "1996-11-12 00:05:00"

        t = time2iso(str2time("11-12-96  1:05PM"))#; print t
        assert t == "1996-11-12 13:05:00"

        t = time2iso(str2time("11-12-96 12:05PM"))#; print t
        assert t == "1996-11-12 12:05:00"

        t = str2time("2000-01-01 00:00:01.234")
        #print "FRAC %s = %s" % (t, time2iso(t))
        assert abs((t - int(t)) - 0.234) < 0.000001

        a = time2iso()
        b = time2iso(500000)
        #print "LOCAL %s  %s" % (a, b)
        az = time2isoz()
        bz = time2isoz(500000)
        #print "GMT   %s %s" % (az, bz)

        for text in (a, b):
            assert re.search(r"^\d{4}-\d\d-\d\d \d\d:\d\d:\d\d$", text)
        for text in (az, bz):
            assert re.search(r"^\d{4}-\d\d-\d\d \d\d:\d\d:\d\dZ$", text)

        # Test the parse_date interface

        d = parse_date("Jan 1 2001")
        assert d.pop() is None and d == [2001, 1, 1, 0, 0, 0.0]

        # This test will break around year 2070
        #if parse_date("03-Feb-20") != "2020-02-03 00:00:00":
        assert parse_date("03-Feb-20") == [2020, 2, 3, 0, 0, 0.0, None]

        # This test will break around year 2048
        #if parse_date("03-Feb-98") != "1998-02-03 00:00:00":
        assert parse_date("03-Feb-98") == [1998, 2, 3, 0, 0, 0.0, None]
