"""Tests for ClientCookie._HeadersUtil."""

from unittest import TestCase

class HeaderTests(TestCase):
    def test(self):
        from ClientCookie._HeadersUtil import split_header_words, join_header_words

        s_tests = [
           ("foo"                     , "foo"),
           ("foo=bar"                 , "foo=bar"),
           ("   foo   "               , "foo"),
           ("foo="                    , 'foo=""'),
           ("foo=bar bar=baz"         , "foo=bar; bar=baz"),
           ("foo=bar;bar=baz"         , "foo=bar; bar=baz"),
           ('foo bar baz'             , "foo; bar; baz"),
           (r'foo="\"" bar="\\"'      , r'foo="\""; bar="\\"'),
           ('foo,,,bar'               , 'foo, bar'),
           ('foo=bar,bar=baz'         , 'foo=bar, bar=baz'),

           ('text/html; charset=iso-8859-1',
            'text/html; charset="iso-8859-1"'),

           ('foo="bar"; port="80,81"; discard, bar=baz',
            'foo=bar; port="80,81"; discard, bar=baz'),

           (r'Basic realm="\"foo\\\\bar\""',
            r'Basic; realm="\"foo\\\\bar\""')
           ]

        #print "split_header_words() tests"
        for test in s_tests:
            arg, expect = test

            input = split_header_words([arg])
            res = join_header_words(input)
            assert res == expect
            if res != expect:
                print "\nUnexpected result: '%s'" % (res,)
                print "         Expected: '%s'" % (expect,)
                print "  when parsing '%s'" % (arg,)
                print "  input was", input


        #print "Extra tests"
        # some extra tests
        assert join_header_words([[("foo", None), ("bar", "baz")]]) == \
               "foo; bar=baz"

        assert join_header_words([[]]) == ""
